#!/bin/bash

queryDatabase () {
  sqlite3 /data/freenas-v1.db
}

getVolumeCount () {
  local VOL_NAME=$1

  queryDatabase <<- EOT
	SELECT COUNT(*)
	FROM storage_volume
	WHERE vol_encrypt > 0
	  AND vol_name    = '$VOL_NAME';
	EOT
}

volumeExist () {
  local VOL_NAME=$1
  local VOL_COUNT=$(getVolumeCount "$VOL_NAME")
  [[ 1 -eq "$VOL_COUNT" ]] && return 0 || return 1
}

getEncryptedDevices () {
  local VOL_NAME=$1

  queryDatabase <<- EOT
	SELECT d.disk_name,
	       d.disk_serial
	FROM storage_disk          d,
	     storage_encrypteddisk e,
	     storage_volume        v
	WHERE e.encrypted_disk_id   = d.id
	  AND e.encrypted_volume_id = v.id
	  AND v.vol_name            = '$VOL_NAME';
	EOT
}

backupMasterKey () {
  local VOL_NAME=$1
  local KEY_PIPE=$2

  if [[ ! -e "$KEY_PIPE" ]]
  then
    mkfifo -m 0600 "$KEY_PIPE"
    geli backup "$VOL_NAME" "$KEY_PIPE"
    rm "$KEY_PIPE"
  fi

  [[ -p "$KEY_PIPE" ]] && rm "$KEY_PIPE"
}

echoEncryptionKey () {
  local VOL_NAME=$1

  queryDatabase <<- EOT
	SELECT vol_encryptkey
	FROM storage_volume
	WHERE vol_name = '$VOL_NAME';
	EOT
}

encryptFromPipe () {
  local PIPE=$1
  local OUTFILE=$2
  local VOL_NAME=$3

  if [[ ! -e "$OUTFILE" ]]
  then
    while [[ ! -p "$PIPE" ]]
    do sleep 1
    done 

    install -m 0600 -o root -g wheel /dev/null "$OUTFILE"
    echoEncryptionKey "$VOL_NAME" | openssl enc -e -a -aes256 -pass stdin -in "$PIPE" -out "$OUTFILE"
  fi
}

printUsage () {
  cat <<- EOT
	Usage: ${0##*/}: <VOLUME>
	
	  <VOLUME> Name of encrypted volume.
	
	EOT
}

main () {
  local VOL_NAME=$1
  local WORK_PATH="/tmp/eli_backup"
  local ARCHIVE="${VOL_NAME}_eli_master_keys.tgz"
  local COL PART2 PART1 KEY_PIPE ENC_FILE ENC_FILE_NAMES ENC_FILES

  if volumeExist "$VOL_NAME"
  then
    install -dm 0700 -o root -g wheel "$WORK_PATH"

    while IFS=\| read -a COL
    do
      PART2="/dev/${COL[0]}p2"
      PART1="/dev/${COL[0]}p1"
      KEY_PIPE="eli_key_${COL[0]}.fifo"
      ENC_FILE="eli_${COL[0]}_${COL[1]}.key.enc"

      if   [[ -c "$PART2" ]]
      then backupMasterKey "$PART2" "${WORK_PATH}/$KEY_PIPE" &
      elif [[ -c "$PART1" ]]
      then backupMasterKey "$PART1" "${WORK_PATH}/$KEY_PIPE" &
      else continue
      fi
 
      encryptFromPipe "${WORK_PATH}/${KEY_PIPE}" "${WORK_PATH}/${ENC_FILE}" "$VOL_NAME" &

      ENC_FILE_NAMES+="${ENC_FILE} "
      ENC_FILES+="${WORK_PATH}/${ENC_FILE} "
    done <<- EOT
	$(getEncryptedDevices "$VOL_NAME")
	EOT

    wait
    install -m 0600 -o root -g wheel /dev/null "${WORK_PATH}/${ARCHIVE}"
    tar -czf "${WORK_PATH}/${ARCHIVE}" -C "$WORK_PATH" $ENC_FILE_NAMES
    rm $ENC_FILES
  else
    printUsage
    return 1
  fi
}

main "$@"
exit $?
